@@ -5,7 +5,7 @@ use 5.014002;
 use strict;
 use warnings;
 
-use version; our $VERSION = version->declare("v1.0.17");
+use version; our $VERSION = version->declare("v1.1.0");
 
 use Carp;
 use List::MoreUtils qw[uniq];
@@ -35,6 +35,9 @@ sub all {
     if ( Zonemaster::Engine::Util::should_run_test( q{connectivity03} ) ) {
         push @results, $class->connectivity03( $zone );
     }
+    if ( Zonemaster::Engine::Util::should_run_test( q{connectivity04} ) ) {
+        push @results, $class->connectivity04( $zone );
+    }
 
     return @results;
 }
@@ -94,11 +97,9 @@ sub metadata {
               ASN_INFOS_ANNOUNCE_IN
               EMPTY_ASN_SET
               ERROR_ASN_DATABASE
-              IPV4_DISABLED
               IPV4_DIFFERENT_ASN
               IPV4_ONE_ASN
               IPV4_SAME_ASN
-              IPV6_DISABLED
               IPV6_DIFFERENT_ASN
               IPV6_ONE_ASN
               IPV6_SAME_ASN
@@ -106,121 +107,168 @@ sub metadata {
               TEST_CASE_START
               )
         ],
+        connectivity04 => [
+            qw(
+              ASN_INFOS_RAW
+              ASN_INFOS_ANNOUNCE_IN
+              CN04_EMPTY_PREFIX_SET
+              CN04_ERROR_PREFIX_DATABASE
+              CN04_IPV4_DIFFERENT_PREFIX
+              CN04_IPV4_SAME_PREFIX
+              CN04_IPV6_DIFFERENT_PREFIX
+              CN04_IPV6_SAME_PREFIX
+              TEST_CASE_END
+              TEST_CASE_START
+              )
+        ],
     };
 } ## end sub metadata
 
 Readonly my %TAG_DESCRIPTIONS => (
     CONNECTIVITY01 => sub {
         __x    # CONNECTIVITY:CONNECTIVITY01
-          'UDP connectivity', @_;
+          'UDP connectivity';
     },
     CONNECTIVITY02 => sub {
         __x    # CONNECTIVITY:CONNECTIVITY02
-          'TCP connectivity', @_;
+          'TCP connectivity';
     },
     CONNECTIVITY03 => sub {
         __x    # CONNECTIVITY:CONNECTIVITY03
-          'AS Diversity', @_;
+          'AS Diversity';
+    },
+    CONNECTIVITY04 => sub {
+        __x    # CONNECTIVITY:CONNECTIVITY04
+          'IP Prefix Diversity';
     },
-
     CN01_IPV4_DISABLED => sub {
         __x    # CONNECTIVITY:CN01_IPV4_DISABLED
           'IPv4 is disabled. No DNS queries are sent to these name servers: "{ns_list}".', @_;
-      },
+    },
     CN01_IPV6_DISABLED => sub {
         __x    # CONNECTIVITY:CN01_IPV6_DISABLED
           'IPv6 is disabled. No DNS queries are sent to these name servers: "{ns_list}".', @_;
-      },
+    },
     CN01_MISSING_NS_RECORD_UDP => sub {
         __x    # CONNECTIVITY:CN01_MISSING_NS_RECORD_UDP
           'Nameserver {ns} responds to a NS query with no NS records in the answer section over UDP.', @_;
-      },
+    },
     CN01_MISSING_SOA_RECORD_UDP => sub {
         __x    # CONNECTIVITY:CN01_MISSING_SOA_RECORD_UDP
           'Nameserver {ns} responds to a SOA query with no SOA records in the answer section over UDP.', @_;
-      },
+    },
     CN01_NO_RESPONSE_NS_QUERY_UDP => sub {
         __x    # CONNECTIVITY:CN01_NO_RESPONSE_NS_QUERY_UDP
           'Nameserver {ns} does not respond to NS queries over UDP.', @_;
-      },
+    },
     CN01_NO_RESPONSE_SOA_QUERY_UDP => sub {
         __x    # CONNECTIVITY:CN01_NO_RESPONSE_SOA_QUERY_UDP
           'Nameserver {ns} does not respond to SOA queries over UDP.', @_;
-      },
+    },
     CN01_NO_RESPONSE_UDP => sub {
         __x    # CONNECTIVITY:CN01_NO_RESPONSE_UDP
           'Nameserver {ns} does not respond to any queries over UDP.', @_;
-      },
+    },
     CN01_NS_RECORD_NOT_AA_UDP => sub {
         __x    # CONNECTIVITY:CN01_NS_RECORD_NOT_AA_UDP
           'Nameserver {ns} does not give an authoritative response on an NS query over UDP.', @_;
-      },
+    },
     CN01_SOA_RECORD_NOT_AA_UDP => sub {
         __x    # CONNECTIVITY:CN01_SOA_RECORD_NOT_AA_UDP
           'Nameserver {ns} does not give an authoritative response on an SOA query over UDP.', @_;
-      },
+    },
     CN01_UNEXPECTED_RCODE_NS_QUERY_UDP => sub {
         __x    # CONNECTIVITY:CN01_UNEXPECTED_RCODE_NS_QUERY_UDP
           'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an NS query over UDP.', @_;
-      },
+    },
     CN01_UNEXPECTED_RCODE_SOA_QUERY_UDP => sub {
         __x    # CONNECTIVITY:CN01_UNEXPECTED_RCODE_SOA_QUERY_UDP
           'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an SOA query over UDP.', @_;
-      },
+    },
     CN01_WRONG_NS_RECORD_UDP => sub {
         __x    # CONNECTIVITY:CN01_WRONG_NS_RECORD_UDP
           'Nameserver {ns} responds with a wrong owner name ({domain_found} instead of {domain_expected}) on NS queries over UDP.', @_;
-      },
+    },
     CN01_WRONG_SOA_RECORD_UDP => sub {
         __x    # CONNECTIVITY:CN01_WRONG_SOA_RECORD_UDP
           'Nameserver {ns} responds with a wrong owner name ({domain_found} instead of {domain_expected}) on SOA queries over UDP.', @_;
-      },
-
+    },
     CN02_MISSING_NS_RECORD_TCP => sub {
         __x    # CONNECTIVITY:CN02_MISSING_NS_RECORD_TCP
           'Nameserver {ns} responds to a NS query with no NS records in the answer section over TCP.', @_;
-      },
+    },
     CN02_MISSING_SOA_RECORD_TCP => sub {
         __x    # CONNECTIVITY:CN02_MISSING_SOA_RECORD_TCP
           'Nameserver {ns} responds to a SOA query with no SOA records in the answer section over TCP.', @_;
-      },
+    },
     CN02_NO_RESPONSE_NS_QUERY_TCP => sub {
         __x    # CONNECTIVITY:CN02_NO_RESPONSE_NS_QUERY_TCP
           'Nameserver {ns} does not respond to NS queries over TCP.', @_;
-      },
+    },
     CN02_NO_RESPONSE_SOA_QUERY_TCP => sub {
         __x    # CONNECTIVITY:CN02_NO_RESPONSE_SOA_QUERY_TCP
           'Nameserver {ns} does not respond to SOA queries over TCP.', @_;
-      },
+    },
     CN02_NO_RESPONSE_TCP => sub {
         __x    # CONNECTIVITY:CN02_NO_RESPONSE_TCP
           'Nameserver {ns} does not respond to any queries over TCP.', @_;
-      },
+    },
     CN02_NS_RECORD_NOT_AA_TCP => sub {
         __x    # CONNECTIVITY:CN02_NS_RECORD_NOT_AA_TCP
           'Nameserver {ns} does not give an authoritative response on an NS query over TCP.', @_;
-      },
+    },
     CN02_SOA_RECORD_NOT_AA_TCP => sub {
         __x    # CONNECTIVITY:CN02_SOA_RECORD_NOT_AA_TCP
           'Nameserver {ns} does not give an authoritative response on an SOA query over TCP.', @_;
-      },
+    },
     CN02_UNEXPECTED_RCODE_NS_QUERY_TCP => sub {
         __x    # CONNECTIVITY:CN02_UNEXPECTED_RCODE_NS_QUERY_TCP
           'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an NS query over TCP.', @_;
-      },
+    },
     CN02_UNEXPECTED_RCODE_SOA_QUERY_TCP => sub {
         __x    # CONNECTIVITY:CN02_UNEXPECTED_RCODE_SOA_QUERY_TCP
           'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an SOA query over TCP.', @_;
-      },
+    },
     CN02_WRONG_NS_RECORD_TCP => sub {
         __x    # CONNECTIVITY:CN02_WRONG_NS_RECORD_TCP
           'Nameserver {ns} responds with a wrong owner name ({domain_found} instead of {domain_expected}) on NS queries over TCP.', @_;
-      },
+    },
     CN02_WRONG_SOA_RECORD_TCP => sub {
         __x    # CONNECTIVITY:CN02_WRONG_SOA_RECORD_TCP
           'Nameserver {ns} responds with a wrong owner name ({domain_found} instead of {domain_expected}) on SOA queries over TCP.', @_;
-      },
-
+    },
+    CN04_ASN_INFOS_ANNOUNCE_IN => sub {
+        __x    # CONNECTIVITY:ASN_INFOS_ANNOUNCE_IN
+          'Name server IP address "{ns_ip}" is announced in prefix "{prefix}".', @_;
+    },
+    CN04_ASN_INFOS_RAW => sub {
+        __x    # CONNECTIVITY:ASN_INFOS_RAW
+          'The ASN data for name server IP address "{ns_ip}" is "{data}".', @_;
+    },
+    CN04_EMPTY_PREFIX_SET => sub {
+        __x    # CONNECTIVITY:CN04_EMPTY_PREFIX_SET
+          'Prefix database returned no information for IP address {ns_ip}.', @_;
+    },
+    CN04_ERROR_PREFIX_DATABASE => sub {
+        __x    # CONNECTIVITY:CN04_ERROR_PREFIX_DATABASE
+          'Prefix database error. No data to analyze for IP address {ns_ip}.', @_;
+    },
+    CN04_IPV4_SAME_PREFIX => sub {
+        __x    # CONNECTIVITY:CN04_IPV4_SAME_PREFIX
+          'The following name server(s) are announced in the same IPv4 prefix ({ip_prefix}): "{ns_list}"', @_;
+    },
+    CN04_IPV4_DIFFERENT_PREFIX => sub {
+        __x    # CONNECTIVITY:CN04_IPV4_DIFFERENT_PREFIX
+          'The following name server(s) are announced in unique IPv4 prefix(es): "{ns_list}"', @_;
+    },
+    CN04_IPV6_SAME_PREFIX => sub {
+        __x    # CONNECTIVITY:CN04_IPV6_SAME_PREFIX
+          'The following name server(s) are announced in the same IPv6 prefix ({ip_prefix}): "{ns_list}"', @_;
+    },
+    CN04_IPV6_DIFFERENT_PREFIX => sub {
+        __x    # CONNECTIVITY:CN04_IPV6_DIFFERENT_PREFIX
+          'The following name server(s) are announced in unique IPv6 prefix(es): "{ns_list}"', @_;
+    },
     ERROR_ASN_DATABASE => sub {
         __x    # CONNECTIVITY:ERROR_ASN_DATABASE
           'ASN Database error. No data to analyze for {ns_ip}.', @_;
@@ -231,31 +279,29 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     IPV4_SAME_ASN => sub {
         __x    # CONNECTIVITY:IPV4_SAME_ASN
-          'All authoritative nameservers have the IPv4 addresses in the same AS set ({asn_list}).', @_;
+          'All authoritative nameservers have their IPv4 addresses in the same AS set ({asn_list}).', @_;
     },
     IPV4_ONE_ASN => sub {
         __x    # CONNECTIVITY:IPV4_ONE_ASN
-          'All authoritative nameservers have the IPv4 addresses in the same AS ({asn}).', @_;
+          'All authoritative nameservers have their IPv4 addresses in the same AS ({asn}).', @_;
     },
     IPV4_DIFFERENT_ASN => sub {
         __x    # CONNECTIVITY:IPV4_DIFFERENT_ASN
-          'At least two IPv4 addresses of the authoritative nameservers are announce by different AS sets. '
-          . 'A merged list of all AS: ({asn_list}).',
-          @_;
+          'At least two IPv4 addresses of the authoritative nameservers are announced by different AS sets. '
+          . 'A merged list of all AS: ({asn_list}).', @_;
     },
     IPV6_SAME_ASN => sub {
         __x    # CONNECTIVITY:IPV6_SAME_ASN
-          'All authoritative nameservers have the IPv6 addresses in the same AS set ({asn_list}).', @_;
+          'All authoritative nameservers have their IPv6 addresses in the same AS set ({asn_list}).', @_;
     },
     IPV6_ONE_ASN => sub {
         __x    # CONNECTIVITY:IPV6_ONE_ASN
-          'All authoritative nameservers have the IPv6 addresses in the same AS ({asn}).', @_;
+          'All authoritative nameservers have their IPv6 addresses in the same AS ({asn}).', @_;
     },
     IPV6_DIFFERENT_ASN => sub {
         __x    # CONNECTIVITY:IPV6_DIFFERENT_ASN
-          'At least two IPv6 addresses of the authoritative nameservers are announce by different AS sets. '
-          . 'A merged list of all AS: ({asn_list}).',
-          @_;
+          'At least two IPv6 addresses of the authoritative nameservers are announced by different AS sets. '
+          . 'A merged list of all AS: ({asn_list}).', @_;
     },
     IPV4_DISABLED => sub {
         __x    # CONNECTIVITY:IPV4_DISABLED
@@ -440,7 +486,7 @@ sub connectivity03 {
 
     my %ips = ( $IP_VERSION_4 => {}, $IP_VERSION_6 => {} );
 
-    foreach my $ns ( @{ Zonemaster::Engine::TestMethods->method4( $zone ) } ) {
+    foreach my $ns ( @{ Zonemaster::Engine::TestMethods->method4and5( $zone ) } ) {
         my $addr = $ns->address;
         $ips{ $addr->version }{ $addr->ip } = $addr;
     }
@@ -559,10 +605,97 @@ sub connectivity03 {
         }
     }
 
-
     return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub connectivity03
 
+sub connectivity04 {
+    my ( $class, $zone ) = @_;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
+
+    my %prefixes;
+
+    foreach my $ns ( @{ Zonemaster::Engine::TestMethods->method4and5( $zone ) } ) {
+        my $ip = $ns->address;
+        my ( $asnref, $prefix, $raw, $ret_code ) = Zonemaster::Engine::ASNLookup->get_with_prefix( $ip );
+
+        if ( defined $ret_code and ( $ret_code eq q{ERROR_ASN_DATABASE} or $ret_code eq q{EMPTY_ASN_SET} ) ) {
+            if ( $ret_code eq 'ERROR_ASN_DATABASE' ) {
+                $ret_code = 'CN04_ERROR_PREFIX_DATABASE';
+            }
+            elsif ( $ret_code eq 'EMPTY_ASN_SET' ) {
+                $ret_code = 'CN04_EMPTY_PREFIX_SET';
+            }
+
+            push @results, info( $ret_code => { ns_ip => $ip->short } );
+        }
+        else {
+            if ( $raw ) {
+                push @results,
+                  info(
+                    CN04_ASN_INFOS_RAW => {
+                        ns_ip => $ip->short,
+                        data  => $raw,
+                    }
+                  );
+            }
+
+            if ( $prefix ) {
+                my $prefix_str;
+
+                if ( $prefix->version == 4 ) {
+                    $prefix_str = $prefix->prefix;
+                }
+                elsif ( $prefix->version == 6 ) {
+                    $prefix_str = $prefix->short . '/' . $prefix->prefixlen;
+                }
+                else {
+                    next;
+                }
+
+                push @results,
+                  info(
+                    CN04_ASN_INFOS_ANNOUNCE_IN => {
+                        ns_ip  => $ip->short,
+                        prefix => sprintf "%s", $prefix_str,
+                    }
+                  );
+
+                push @{ $prefixes{$prefix->version}{$prefix_str} }, $ns;
+            }
+        }
+    }
+
+    foreach my $ip_version ( sort keys %prefixes ) {
+        my @combined_ns;
+
+        foreach my $prefix ( keys %{ $prefixes{$ip_version} } ) {
+            if ( scalar @{ $prefixes{$ip_version}{$prefix} } == 1 ) {
+                push @combined_ns, @{ $prefixes{$ip_version}{$prefix} };
+            }
+            elsif ( scalar @{ $prefixes{$ip_version}{$prefix} } >= 2 ) {
+                push @results,
+                  info(
+                    "CN04_IPV${ip_version}_SAME_PREFIX" => {
+                        ip_prefix => $prefix,
+                        ns_list => join( q{;}, sort @{ $prefixes{$ip_version}{$prefix} } )
+                    }
+                  );
+            }
+        }
+
+        if ( scalar @combined_ns ) {
+            push @results,
+              info(
+                "CN04_IPV${ip_version}_DIFFERENT_PREFIX" => {
+                    ns_list => join( q{;}, sort @combined_ns )
+                }
+              );
+        }
+    }
+
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
+} ## end sub connectivity04
+
 1;
 
 =head1 NAME
@@ -612,6 +745,10 @@ Verify nameservers TCP port 53 reachability.
 
 Verify that all nameservers do not belong to the same AS.
 
+=item connectivity04($zone)
+
+Verify that name servers are not announced in the same IP prefix.
+
 =back
 
 =cut
